/*
 * Copyright (c) 2022 NIBIO <http://www.nibio.no/>. 
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */
package no.nibio.model.maizepheno;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.util.List;
import no.nibio.vips.entity.ModelConfiguration;
import no.nibio.vips.entity.Result;
import no.nibio.vips.model.ModelId;
import no.nibio.vips.util.test.WeatherDataFileReader;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.*;

/**
 *
 * @author treinar
 */
public class MaizePhenologyModelTest {
    
    public MaizePhenologyModelTest() {
    }

    @org.junit.jupiter.api.BeforeAll
    public static void setUpClass() throws Exception {
    }

    @org.junit.jupiter.api.AfterAll
    public static void tearDownClass() throws Exception {
    }

    @org.junit.jupiter.api.BeforeEach
    public void setUp() throws Exception {
    }

    @org.junit.jupiter.api.AfterEach
    public void tearDown() throws Exception {
    }
    
    

    /**
     * Test of getResult method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetResult() throws Exception {
        System.out.println("getResult");
        MaizePhenologyModel instance = new MaizePhenologyModel();
       
        // REQUIRED
        WeatherDataFileReader wfr = new WeatherDataFileReader();
        // Weather data files can be placed in ("src/test/resources")
        //ModelConfiguration config = wfr.getModelConfigurationWithWeatherData("/weather_data/hourly_avg_temps_01_station_id_TA00644.json", MaizePhenologyModel.MODEL_ID.toString());
        ModelConfiguration config = wfr.getModelConfigurationWithWeatherData("/weather_data/hourly_avg_temps_01_station_id_TA00644.json", MaizePhenologyModel.MODEL_ID.toString());
        // The timezone is used to set daily temperatures and biofix date correctly
        config.setConfigParameter("timeZone", "GMT+01:00");
        // The date for when to start calculating heat sums
        config.setConfigParameter("sowingDate", "2020-11-18");
        
        //System.out.println(config.toJSON());
        
        instance.setConfiguration(config);
        List<Result> result = instance.getResult();
        
        //result.forEach(r->System.out.println(r));
        
        /*
        ObjectMapper om = new ObjectMapper();
        try {
            System.out.println(om.writeValueAsString(result));
        } catch (JsonProcessingException ex) {
            fail(ex);
        }
        */
        
        assertNotNull(result);
        
        //result.forEach(r->System.out.println(r));
        
        // Testing with hourly min and max temps
        config = wfr.getModelConfigurationWithWeatherData("/weather_data/hourly_min_max_Apelsvoll_Norway.json", MaizePhenologyModel.MODEL_ID.toString());
        
        // The timezone is used to set daily temperatures and biofix date correctly
        config.setConfigParameter("timeZone", "Europe/Oslo");
        // The date for when to start calculating heat sums
        config.setConfigParameter("sowingDate", "2021-06-02");
        
        //System.out.println(config.toJSON());
        
        instance.setConfiguration(config);
        result = instance.getResult();
        
        assertNotNull(result);
        
        //result.forEach(r->System.out.println(r));
        
        // Testing with daily min and max temps
        config = wfr.getModelConfigurationWithWeatherData("/weather_data/daily_min_max_Apelsvoll_Norway.json", MaizePhenologyModel.MODEL_ID.toString());
        
        // The timezone is used to set daily temperatures and biofix date correctly
        config.setConfigParameter("timeZone", "Europe/Oslo");
        // The date for when to start calculating heat sums
        config.setConfigParameter("sowingDate", "2021-06-02");
        
        //System.out.println(config.toJSON());
        
        instance.setConfiguration(config);
        result = instance.getResult();
        assertNotNull(result);
        
        //result.forEach(r->System.out.println(r));
    }

    /**
     * Test of getModelId method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetModelId() {
        System.out.println("getModelId");
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String expResult = "MAIZEPHENO";
        ModelId result = instance.getModelId();
        assertEquals(expResult, result.toString());
        
    }

    /**
     * Test of getModelName method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetModelName_0args() {
        System.out.println("getModelName");
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String expResult = "Maize Phenology Model";
        String result = instance.getModelName();
        assertNotNull(result);
        assertTrue(!result.isBlank());
    }

    /**
     * Test of getModelName method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetModelName_String() {
        System.out.println("getModelName");
        String language = "nb";
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String expResult = "Mais-fenologimodell";
        String result = instance.getModelName(language);
        assertEquals(expResult, result);
        
    }

    /**
     * Test of getLicense method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetLicense() {
        System.out.println("getLicense");
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String result = instance.getLicense();
        assertNotNull(result);
        
    }

    /**
     * Test of getCopyright method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetCopyright() {
        System.out.println("getCopyright");
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String expResult = "(c) 2022 NIBIO (http://www.nibio.no/). Contact: post@nibio.no";
        String result = instance.getCopyright();
        assertEquals(expResult, result);
        
    }

    /**
     * Test of getModelDescription method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetModelDescription_0args() {
        System.out.println("getModelDescription");
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String result = instance.getModelDescription();
        assertNotNull(result);
    }

    /**
     * Test of getModelDescription method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetModelDescription_String() {
        System.out.println("getModelDescription");
        String language = "nb";
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String result = instance.getModelDescription(language);
        assertNotNull(result);
    }

    /**
     * Test of getWarningStatusInterpretation method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetWarningStatusInterpretation_0args() {
        System.out.println("getWarningStatusInterpretation");
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String result = instance.getWarningStatusInterpretation();
        assertNotNull(result);
        
    }

    /**
     * Test of getWarningStatusInterpretation method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetWarningStatusInterpretation_String() {
        System.out.println("getWarningStatusInterpretation");
        String language = "nb";
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String result = instance.getWarningStatusInterpretation(language);
        assertNotNull(result);
    }

    /**
     * Test of getModelUsage method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetModelUsage_0args() {
        System.out.println("getModelUsage");
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String result = instance.getModelUsage();
        assertNotNull(result);
        
    }

    /**
     * Test of getModelUsage method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetModelUsage_String() {
        System.out.println("getModelUsage");
        String language = "";
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String result = instance.getModelUsage(language);
        assertNotNull(result);
       
    }

    /**
     * Test of getSampleConfig method, of class MaizePhenologyModel.
     */
    @org.junit.jupiter.api.Test
    public void testGetSampleConfig() {
        System.out.println("getSampleConfig");
        MaizePhenologyModel instance = new MaizePhenologyModel();
        String result = instance.getSampleConfig();
        assertTrue(result.indexOf("sowingDate") > 0);

    }

   
}
